from mate import generators
from mate.stats import FeatureStatistics, FeatureType


def test_create_statistics_feature(df):
    for name, feature_series in df.iteritems():
        feature_stats = generators._create_statistics_feature(feature_series)
        assert isinstance(feature_stats, FeatureStatistics)


def test_infer_feature_type(df):
    mapping = {
        "age": FeatureType.INTEGER,
        "sex": FeatureType.STRING,
        "bmi": FeatureType.FRACTION,
        "children": FeatureType.INTEGER,
        "smoker": FeatureType.STRING,
        "region": FeatureType.STRING,
    }

    for name, feature_series in df.iteritems():
        feature_type = generators._infer_feature_type(feature_series)
        assert feature_type == mapping[name]



import pytest

from mate.checks import is_out_of_bounds, is_outlier


@pytest.mark.parametrize(
    "feature_value, mean, std_dev, expected",
    [
        (31.0, 39.20702541106129, 14.049960379216154, False),
        (2.0, 1.0949177877429, 1.205492739781914, False),
        (96.0, 39.20702541106129, 14.049960379216154, True),
    ],
)
def test_is_outlier(feature_value, mean, std_dev, expected):
    assert is_outlier(feature_value, mean, std_dev) == expected


@pytest.mark.parametrize(
    "feature_value, lower_bound, upper_bound, expected",
    [
        (31.0, 18.0, 64.0, False),
        (2.0, 0.0, 5.0, False),
        (-1.0, 18.0, 64.0, True),
        (2.0, 0.0, 5.0, False),
    ],
)
def test_is_out_of_bounds(feature_value, lower_bound, upper_bound, expected):
    assert is_out_of_bounds(feature_value, lower_bound, upper_bound) == expected



def is_out_of_bounds(
    feature_value: float, lower_bound: float, upper_bound: float
) -> bool:
    """
    Checks if the feature_value is outside of the lower or upper bounds.
    """

    return (feature_value < lower_bound) or (feature_value > upper_bound)


def is_outlier(
    feature_value: float, mean: float, std_dev: float, outlier_cutoff: float = 4.0
) -> bool:
    """
    Checks if the Median Absolute Deviation (MAD) is greater than the outlier_cutoff (defaults to 4.0).
    """

    return (abs(feature_value - mean) / std_dev) > outlier_cutoff



from dataclasses import dataclass, field
from enum import Enum
from typing import List, Optional


class FeatureType(Enum):
    FRACTION = "fraction"
    INTEGER = "integer"
    STRING = "string"
    UNKNOWN = "unknown"


@dataclass
class CommonStatistics:
    num_present: int
    num_missing: int


@dataclass
class NumericalStatistics:
    common: CommonStatistics
    mean: float
    sum: float
    std_dev: float
    min: float
    max: float


@dataclass
class StringStatistics:
    common: CommonStatistics
    distinct_count: int


@dataclass
class FeatureStatistics:
    name: str
    inferred_type: str
    numerical_statistics: Optional[NumericalStatistics] = None
    string_statistics: Optional[StringStatistics] = None


@dataclass
class Statistics:
    item_count: int
    version: int = 0
    features: List[FeatureStatistics] = field(default_factory=list)



import os
import pathlib

import pandas as pd  # type: ignore

from mate.db import Inference, Mate
from mate.run import RunningMate

BASE = pathlib.Path(__file__).parent.parent.absolute()


def test_running_mate_no_alerts(baseline_stats):
    os.environ["TESTING"] = "1"

    assert Mate.select().count() == 1

    alert_targets = []
    df = pd.read_csv(BASE.joinpath("data/insurance_infer.csv"), sep=",")

    test_mate = RunningMate(
        "insurance", 1, df, alert_targets, should_save_all_feature_values=True
    )

    assert Inference.select().count() == 1
    assert test_mate.mate_name == "insurance"
    assert test_mate.should_save_all_feature_values
    assert len(test_mate._check_statistics(df)) == 0


def test_running_mate_with_alerts(baseline_stats):
    os.environ["TESTING"] = "1"

    assert Mate.select().count() == 1

    alert_targets = []
    df = pd.read_csv(BASE.joinpath("data/insurance_infer.csv"), sep=",")
    df["age"] = -1

    test_mate = RunningMate(
        "insurance", 1, df, alert_targets, should_save_all_feature_values=True
    )

    assert Inference.select().count() == 2
    assert test_mate.mate_name == "insurance"
    assert test_mate.should_save_all_feature_values
    assert len(test_mate._check_statistics(df)) == 1



import setuptools
from pathlib import Path

BASE = Path(__file__).parent
long_description = (BASE / "README.md").read_text()


setuptools.setup(
    name="running-mate",
    version="0.0.5",
    author="Michael Herman",
    author_email="michael@mherman.org",
    description="Lightweight MLOps framework.",
    url="https://github.com/mjhea0/running-mate",
    packages=setuptools.find_packages(),
    install_requires=[
        "dacite==1.6.0",
        "numpy==1.21.3",
        "pandas==1.3.4",
        "peewee==3.14.8",
        "requests==2.26.0",
    ],
    long_description=long_description,
    long_description_content_type="text/markdown"
)



import os
import pathlib

import pandas as pd  # type: ignore
import pytest

from mate.db import connect_db, version_or_create_mate
from mate.generators import generate_baseline_stats

BASE = pathlib.Path(__file__).parent.parent.absolute()


@pytest.fixture(scope="module")
def df():
    return pd.read_csv(BASE.joinpath("data/insurance.csv"), sep=",")


@pytest.fixture(scope="module")
def baseline_stats(df):
    os.environ["TESTING"] = "1"
    new_df = df.copy()
    X = new_df.drop(["charges"], axis=1)

    connect_db()
    version_or_create_mate("insurance")

    return generate_baseline_stats(X, "insurance")



import os

from mate.db import (
    Feature,
    Mate,
    NumericalStats,
    StringStats,
    connect_db,
    version_or_create_mate,
)
from mate.generators import generate_baseline_stats


def test_generate_baseline_stats(df):
    os.environ["TESTING"] = "1"
    connect_db()

    assert Mate.select().count() == 0
    assert Feature.select().count() == 0
    assert NumericalStats.select().count() == 0
    assert StringStats.select().count() == 0

    new_df = df.copy()
    X = new_df.drop(["charges"], axis=1)

    version_or_create_mate("insurance")
    assert Mate.select().count() == 1

    generate_baseline_stats(X, "insurance")
    assert Feature.select().count() == 6
    assert NumericalStats.select().count() == 3
    assert StringStats.select().count() == 3

    # create new version
    version_or_create_mate("insurance")
    assert Mate.select().count() == 2

    generate_baseline_stats(X, "insurance")
    assert Feature.select().count() == 12
    assert NumericalStats.select().count() == 6
    assert StringStats.select().count() == 6
